import web
import os
import logging
import jsonpickle
import mimerender
import formencode.api
import lucene
from errors import NotFound, WebException
from util.web import render_mako, jsonpwrap

MIME_INPUT_KEY = 'format'
mimerender.global_default = 'xml'
mimerender.global_override_input_key = MIME_INPUT_KEY
mimerender.global_charset = 'utf-8'

xml_render = render_mako(_extension = 'xml',
    directories=[os.path.join(os.path.dirname(__file__),
                              'templates/xml').replace('\\','/'),],
    input_encoding='utf-8', output_encoding='utf-8')

tsv_render = render_mako(_extension = 'tsv',
    directories=[os.path.join(os.path.dirname(__file__),
                              'templates/tsv').replace('\\','/'),],
    input_encoding='utf-8', output_encoding='utf-8')

m3u_render = render_mako(_extension = 'm3u',
    directories=[os.path.join(os.path.dirname(__file__),
                              'templates/m3u').replace('\\','/'),],
    input_encoding='utf-8', output_encoding='utf-8')

#kml_render = render_mako(_extension = 'kml',
#    directories=[os.path.join(os.path.dirname(__file__),
#                              'templates/kml').replace('\\','/'),],
#    input_encoding='utf-8', output_encoding='utf-8')

xspf_render = render_mako(_extension = 'xspf',
    directories=[os.path.join(os.path.dirname(__file__),
                              'templates/xspf').replace('\\','/'),],
    input_encoding='utf-8', output_encoding='utf-8')

def json_render(exception):
    return jsonpickle.encode(
        dict(
            error=dict(
                type=exception.__module__ + "." + exception.__class__.__name__,
                message=exception.message
            )
        )
    )

@mimerender.mimerender(
    json = json_render,
    jsonp = jsonpwrap(json_render),
    txt = tsv_render.error,
    xml = xml_render.error,
    tsv = tsv_render.error,
    #kml = kml_render.error,
    m3u = m3u_render.error,
    xspf = xspf_render.error,
)
def _render(exception):
    return dict(exception=exception)

def decorator(wrapped):
    """decorator that converts exceptions to more RESTful (web.py) ones."""
    def wrapper(*args, **kwargs):
        logger = logging.getLogger('gracenote.rest-ws.exceptionmapper')
        try:
            result = wrapped(*args, **kwargs)
            return result
        except NotFound, e:
            logger.exception('NotFound exception')
            data = _render(e)
            raise web.HTTPError('404 Not Found', {}, data)
        except WebException, e:
            logger.exception('Web exception')
            data = _render(e)
            raise web.HTTPError('500 Internal Error', {}, data)
        except formencode.api.Invalid, e:
            logger.exception('Validation exception')
            data = _render(e)
            raise web.HTTPError('400 Bad Request', {}, data)
        except lucene.JavaError, e:
            logger.exception('Lucene exception')
            data = _render(e)
            raise web.HTTPError('400 Bad Request', {}, data)
        except mimerender.MimeRenderException, e:
            logger.exception('mimerender exception')
            data = xml_render.error(e)
            raise web.HTTPError('406 Not Acceptable',
                                {'Content-type':
                                 'application/xml; charset=utf-8'}, data)
    return wrapper
