import sys
import os
import re
import web
import jsonpickle
import mimerender
import logging
import logging.config
import datetime
from formencode import validators, Schema
try: 
    import json
except ImportError: 
    import simplejson as json
try:
    mimerender.register_mime('jsonp', ('application/javascript',))
except mimerender.MimeRenderException:
    pass

from operator import itemgetter

#Utils
from util.web import render_mako, jsonpwrap, urlencode
from util import pyconfig

#Errors, Validators and Exceptions
from exceptionmapper import decorator as mapexceptions
import errors.custom_validators
from errors import NotFound

################################################################################
# User
from util.lastfm import get_top_artists
from util.echonest import get_songs
username = sys.argv[2]
print username
artists = get_top_artists(username)
print artists
TYPES = ['relaxed', 'danceable', 'happy', 'sad']
for mbid, playcounts, artist in artists:
    for type in TYPES:
        print get_songs(artist, type)

################################################################################
# initialize logging
try:
    LOGGING_CFGS = ['logging.ini']
except KeyError:
    LOGGING_CFGS = [
        'logging.ini',
        '/etc/gracenote/logging.ini',
        os.path.expanduser('~/.gracenote/logging.ini'),
        'logging.ini'
    ]
read_files = []
for filename in LOGGING_CFGS:
    try:
        logging.config.fileConfig(filename)
        read_files.append(filename)
    except NoSectionError: 
        pass

module_logger = logging.getLogger('gracenote.rest-ws.service')
if read_files:
    module_logger.debug('logging config using: %s' % read_files)
else:
    module_logger.warn('logging config not found (tried with %s)' % LOGGING_CFGS)

module_logger = logging.getLogger('gracenote.webapp.service')

################################################################################
# load core configuration
try:
    CONFIG_FILES = [os.environ['GRACENOTE_RESTWS_CONFIG']]
except KeyError:
    CONFIG_FILES = [
        'config/config.py',
        os.path.expanduser('~/.gracenote/config.py'),
        'config.py',
    ]
config_found = False
for config_file in CONFIG_FILES:
    if os.path.exists(config_file):
        config_found = True
        break
if not config_found: 
    raise Exception('configuration not found (tried with %s)' % CONFIG_FILES)

################################################################################
# renderers
xml_render = render_mako(_extension = 'xml',
    directories=[os.path.join(os.path.dirname(__file__),
                              'templates/xml').replace('\\','/'),],
    input_encoding='utf-8', output_encoding='utf-8')

tsv_render = render_mako(_extension = 'tsv',
    directories=[os.path.join(os.path.dirname(__file__),
                              'templates/tsv').replace('\\','/'),],
    input_encoding='utf-8', output_encoding='utf-8')

xspf_render = render_mako(_extension = 'xspf',
    directories=[os.path.join(os.path.dirname(__file__),
                              'templates/xspf').replace('\\','/'),],
    input_encoding='utf-8', output_encoding='utf-8')

m3u_render = render_mako(_extension = 'm3u',
    directories=[os.path.join(os.path.dirname(__file__),
                              'templates/m3u').replace('\\','/'),],
    input_encoding='utf-8', output_encoding='utf-8')

#kml_render = render_mako(_extension = 'kml',
#    directories=[os.path.join(os.path.dirname(__file__),
#                              'templates/kml').replace('\\','/'),],
#    input_encoding='utf-8', output_encoding='utf-8')

json_render = lambda **args: jsonpickle.encode(args) #TODO if int, then fails!  :-(

################################################################################
# mimerender setup
MIME_INPUT_KEY = 'format'
mimerender.global_default = 'xml'
mimerender.global_override_input_key = MIME_INPUT_KEY
mimerender.global_charset = 'utf-8'

################################################################################
# configuration file
config = pyconfig.read(config_file)

################################################################################
# web.py (auto_)application
app = web.auto_application()

################################################################################
# Generic Items
MODULE_NAMES = os.listdir('./entities')
MODULES = list(set([module.split('.')[0] for module in MODULE_NAMES if not module.startswith('__')]))
_entities = dict()

for module in MODULES:
    try: 
        m = __import__('entities.%ss' % module)
        c = eval("m.%s.%s(config['collections']['%s'])" % (module + 's', module.title() + 's', module))
        _entities[module] = c
    except Exception, e: 
        #print 'error', e
        continue
#print _entities

################################################################################
# web input processing
API_KEY_INPUT_KEY = 'MTL-ftw!'

def input():
    d = dict(web.input())
    d.pop(MIME_INPUT_KEY, None)
    d.pop(API_KEY_INPUT_KEY, None)
    d.pop('callback', None)
    return d

################################################################################
# pages


class status(app.page):
    #path = '(?:/([^/]+))?/status'
    path = '(?:/([^/]+))?/((?:[^/.]+)(?:/[^/.]+)*)/status'

    @mapexceptions
    @mimerender.mimerender(
        json = json_render,
        jsonp = jsonpwrap(json_render),
        txt = tsv_render.results,
        xml = xml_render.results,
        tsv = tsv_render.results,
    )
    def GET(self, entity_type, id):
        #sanitize get-parameters
        params = input()
        response = []

        try:
            response = _entities[entity_type].status(id)
        except Exception, e:
            print 'error', e
            pass
        if not response: raise(NotFound)
        return {'response': response}

class set(app.page):
    path = '(?:/([^/]+))?/((?:[^/.]+)(?:/[^/.]+)*)/set'

    @mapexceptions
    @mimerender.mimerender(
        json = json_render,
        jsonp = jsonpwrap(json_render),
        txt = tsv_render.results,
        xml = xml_render.results,
        tsv = tsv_render.results,
    )
    def GET(self, entity_type, id):
        #sanitize get-parameters
        params = input()
        print 'PARAMS', params
        attention = eval(params['attention'])
        meditation = eval(params['meditation'])
        response = []

        try:
            response = _entities[entity_type].set(id, attention, meditation)
        except Exception, e:
            print 'error', e
            pass
        if not response: raise(NotFound)
        return {'response': response}

class img(app.page):
    #path = '(?:/([^/]+))?/status'
    path = '(?:/([^/]+))?/((?:[^/.]+)(?:/[^/.]+)*)/img'

    @mapexceptions
    @mimerender.mimerender(
        json = json_render,
        jsonp = jsonpwrap(json_render),
        txt = tsv_render.results,
        xml = xml_render.results,
        tsv = tsv_render.results,
    )
    def GET(self, entity_type, id):
        #sanitize get-parameters
        params = input()
        response = []

        try:
            img = _entities[entity_type].img(id)
            print img
            print 'static/img/%s.png'%img
            return web.seeother('/static/img/%s.png'%img)
        except Exception, e:
            print 'error', e
            pass
        if not response: raise(NotFound)

class entities(app.page):
    """Get one entity info
    """
    path = '(?:/([^/]+))?/((?:[^/.]+)(?:/[^/.]+)*)'
    
    class validator(Schema):
        fetch_metadata = validators.UnicodeString(not_empty=True,
                                                  if_missing='_default')
    
    @mapexceptions
    @mimerender.mimerender(
        json = json_render,
        jsonp = jsonpwrap(json_render),
        txt = tsv_render.results,
        xml = xml_render.results,
        tsv = tsv_render.results,
    )
    def GET(self, entity_type, id):
        params = input()
        params = self.validator.to_python(params)
        
        fetch_metadata = params.pop('fetch_metadata', '_default')
        try:
            response = _entities[entity_type].status(id)
        except Exception, e:
            print e
            response = None
        if not response: raise(NotFound)
        return {'response': response}

web.config.debug = True
if __name__ == '__main__': app.run()

