try:
    import lucene as pylucene
    from lucene import PythonAnalyzer
except:
    import PyLucene as pylucene
    class PythonAnalyzer: pass

import analyzer

class indexer:
    def __init__(self, path):
        self.index_path = path
    
    @staticmethod
    def index_exists(index_path):
        return pylucene.IndexReader.indexExists(pylucene.FSDirectory.getDirectory(index_path, False))
    
    @staticmethod
    def optimize(index_path):
        i = indexer(index_path)
        i.open_index()
        i.close_index()

    def open_index(self, create = False, anal = None, optimize = True):
        store = pylucene.FSDirectory.getDirectory(self.index_path, create)
        if not anal:
            anal = analyzer.basic_analyzer()
        self.idxwriter = pylucene.IndexWriter(store, False, anal, create)
        if optimize:
            print 'Applying optimization'
            #self.idxwriter.setMergeFactor(1000)
            self.idxwriter.setRAMBufferSizeMB(1024.0)
            #self.idxwriter.setMaxMergeDocs(100000)

    def close_index(self):
        self.idxwriter.optimize()
        self.idxwriter.close()   

    def add(self, data_dict, boost = None):
        doc = pylucene.Document()
        for field_key, value in data_dict.iteritems():
            field_value, is_stored, is_tokenized = value
            store = pylucene.Field.Store.YES
            if not is_stored: store = pylucene.Field.Store.NO
            tokenize = pylucene.Field.Index.TOKENIZED
            if not is_tokenized: tokenize = pylucene.Field.Index.UN_TOKENIZED
            if isinstance(field_value, list):
                for field in field_value:
                    doc.add(pylucene.Field(field_key, unicode(field), store, tokenize))
            else:
                doc.add(pylucene.Field(field_key, unicode(field_value), store, tokenize))
        if boost:
            doc.setBoost(boost)
        self.idxwriter.addDocument(doc)
        
   
    def add_list(self, data_list, boost = None):
        doc = pylucene.Document()
        for field_key, field_value, is_stored, is_tokenized in data_list:
            store = pylucene.Field.Store.YES
            if not is_stored: store = pylucene.Field.Store.NO
            tokenize = pylucene.Field.Index.TOKENIZED
            if not is_tokenized: tokenize = pylucene.Field.Index.UN_TOKENIZED                   
            if isinstance(field_value, list):
                for field in field_value:
                    doc.add(pylucene.Field(field_key, unicode(field), store, tokenize))
            else:
                doc.add(pylucene.Field(field_key, unicode(field_value), store, tokenize))

        if boost:
            doc.setBoost(float(boost))
        self.idxwriter.addDocument(doc)

    def merge_indexes(self, index_path_list):
        '''
        self.idxwriter.setMergeFactor(1000)
        self.idxwriter.setRAMBufferSizeMB(50)
        '''
        dir_list = []
        for p in index_path_list:
            dir_list.append(pylucene.FSDirectory.getDirectory(p))
        self.idxwriter.addIndexes(dir_list)

