try:
    import lucene as pylucene
    from lucene import PythonAnalyzer
except:
    import PyLucene as pylucene
    class PythonAnalyzer: pass

import analyzer as Anal
from Levenshtein import ratio
from operator import itemgetter

class Search(object):
    def __init__(self, indexName, analyzer = None):
        if not analyzer:
            self.analyzer = Anal.basic_analyzer()
        else:
            self.analyzer = analyzer
        try:
            self.index = pylucene.IndexSearcher(pylucene.FSDirectory.getDirectory(indexName, False))
        except ValueError:
            raise NoSuchIndexError
        
        self.new_match = pylucene.VERSION.startswith('2.3')
        
    @staticmethod
    def init():
        try:
            print ("Initializing the JVM...")
            pylucene.initVM(pylucene.CLASSPATH)
            print ("Done initializing the JVM...")
        except Exception,e:
            print ("Could not initialize the JVM for lucene(%s)"%e)

    def iterread(self):
        for it in range(self.index.maxDoc()):
            doc = self.index.doc(it)
            d = {}
            if self.new_match:
                for field_d in doc.getFields():
                    field = pylucene.Field.cast_(field_d)

                    if d.has_key(field.name()):
                        if isinstance(d[field.name()][0], list):
                            d[field.name()] = [d[field.name()][0] + [field.stringValue()], field.isStored(), field.isTokenized() ]
                        else:
                            d[field.name()] = [[d[field.name()][0], field.stringValue()], field.isStored(), field.isTokenized() ]
                    else:
                        d[field.name()]=[ field.stringValue(), field.isStored(), field.isTokenized() ]
            else:
                for field in doc.fields():
                    d[field.name()]= [ doc.get(field.name()), field.isStored(), field.isTokenized() ]
            
            yield d
    
    def close(self):
        self.index.close()

    def itermatch_old(self, query, field, maxHits):
        parser = pylucene.QueryParser(field, self.analyzer)
        parsedQuery = parser.parse(query)
        if parsedQuery.isBooleanQuery():
            parsedQuery = parsedQuery.toBooleanQuery()
            parsedQuery.setMaxClauseCount(4096)
        else:
            pass
        #print parsedQuery
        hits = list(self.index.search(parsedQuery))
        if not hits:
            return
        for hit in hits[:maxHits]:
            doc = hit.getDocument()
            d = {}
            for field in doc.fields():
                d[field.name()]=doc.get(field.name())
                d['relevance'] = hit.getScore()
            yield d

    def itermatch(self, query, field, maxHits):
        if isinstance(query,pylucene.BooleanQuery):
            hits = self.index.search(query)
        else:
            parser = pylucene.QueryParser(field, self.analyzer)
            parsedQuery = parser.parse(query)
            hits = self.index.search(parsedQuery)
        if hits.length() == 0:
            return
        if maxHits < 0:
            r = hits.length()
        else:
            r = min(hits.length(),maxHits)
        results = []
        for i in range(r):
            doc = hits.doc(i)
            d = {}
            d['relevance'] = hits.score(i)
            refine_relevance = query.find(':') == -1 or query.find('name:') != -1
            for field_d in doc.getFields():
                field = pylucene.Field.cast_(field_d)
                if d.has_key(field.name()):
                    if isinstance(d[field.name()], list):
                        d[field.name()] = d[field.name()] + [field.stringValue()]
                    else:
                        d[field.name()] = [d[field.name()], field.stringValue()]
                else:
                    d[field.name()]=field.stringValue()
                if refine_relevance and field.name() == 'name': #and d['relevance'] == 1.0:
                    #d['relevance'] = min(d['relevance'], ratio(d[field.name()].lower(), query.lower()))
                    try:
                        d['relevance'] = ratio(d[field.name()].lower(), query.split('name:')[-1].lower())
                    except:
                        d['relevance'] = ratio(d[field.name()].lower().decode('utf8'), query.split('name:')[-1].lower().decode('utf8'))
            #yield d
            results.append((d, d['relevance']))
        results.sort(key=itemgetter(1), reverse=True)
        results = map(itemgetter(0), results)
        return results

    def match(self, query, field, maxHits):
        if self.new_match:
            res = [ foo for foo in self.itermatch(query, field, maxHits)]
        else:
            res = [ foo for foo in self.itermatch_old(query, field, maxHits)]
        if len(res) == 0:
            return
        return res

    def match_term_query(self, data, field, maxHits):
        query = pylucene.BooleanQuery()
        for key,value in data.iteritems():
            query.add(pylucene.TermQuery(pylucene.Term(key,value)),pylucene.BooleanClause.Occur.MUST)
        return self.match(query, field, maxHits)

        
if __name__ == '__main__':
    import sys,os, codecs, traceback
    from optparse import OptionParser
    import utils
    usage = "usage: %prog [options] index query"
    parser = OptionParser(usage)
    parser.add_option("-l", "--limit", dest="limit", default=1,
                          help="query limit", type="int")    
    
    parser.add_option("-e", "--extra_fields", dest="extra",
                              help="extra fields to show", type="string")

    parser.add_option("-t", "--term-query", dest="term_query", action = "store_true", default=False,
                              help="Based on term query AND. key:val key2:val ...")

    (options, args) = parser.parse_args()      
    if len(args) < 2:
        parser.print_help()
        sys.exit(0)

    utils.init()                      

    s = Search(args[0])
    query = unicode(' '.join(args[1:]), 'utf8')
    
    if options.term_query:
        data = {}
        for dstr in args[1:]:
            key,value = [d.strip() for d in dstr.split(':')]
            data[key] = value
        res = s.match_term_query(data, 'indexable_field',options.limit)
    else:
        res = s.match(query, 'indexable_field',options.limit)

    if not res:
        print 'No match.'
        sys.exit(0)

    #print res
    if options.extra:
        fields = [ field.strip() for field in  options.extra.split(',')]
    else:
        fields = None
    for r in res:
        if r['entity_type'].lower() =='track':
            print 'TRACK: %s by %s (relevance=%.2f)'%(r['track'],r.get('artist',''),r['relevance'])
        elif r['entity_type'].lower() =='artist':
            print 'ARTIST: %s (relevance=%.2f)'%(r['artist'],r['relevance'])
        elif r['entity_type'].lower() =='release':
            print 'RELEASE: %s by %s (relevance=%.2f)'%(r['release'],r.get('artist',''),r['relevance'])
        else:
            print '%s: %s (relevance=%.2f)'%(r['entity_type'],r['name'],r['relevance'])
        if not fields:
            fields = r.keys()
        output_str = ''
        for field in fields:
            value = r.get(field, None)
            if value:
                output_str += '\t%s: %s\n'%(field, value)
        print output_str


