import urllib2
import logging
from errors import NotFound

class LastFmUserNotFound(NotFound): pass

WS_URL = 'http://ws.audioscrobbler.com/1.0/user/%s/top%ss.txt'

def _get_logger():
    return logging.getLogger('rest-ws.util.lastfm')

def get_chart(username, entity_type):
    """Get a chart for a given user.
    
#        >>> mbid, playcount, name = get_chart('martinblech', 'artist')[0]
#        >>> isinstance(mbid, basestring)
#        True
#        >>> isinstance(playcount, int)
#        True
#        >>> isinstance(name, basestring)
#        True
#        >>> get_chart('_i_dont_exist_12345fdsax', 'artist')
#        Traceback (most recent call last):
#          ...
#        LastFmUserNotFound: top artists for user _i_dont_exist_12345fdsax not found
    
    """
    logger = _get_logger()
    logger.debug('getting top %ss for user %s' % (entity_type, username))
    try:
        f = urllib2.urlopen(WS_URL % (username, entity_type))
    except:
        raise LastFmUserNotFound('top %ss for user %s not found' % (entity_type, username))
    if f.code != 200:
        raise LastFmUserNotFound('top %ss for user %s not found' % (entity_type, username))
    results = list()
    for line in f.readlines():
        try:
            split = line.strip().split(',')
            mbid = split[0] or None
            playcount = int(split[1])
            name = ','.join(split[2:]).replace('\xe2\x80\x93', '-')
            results.append((mbid, playcount, name))
        except Exception, e:
            logger.warning("couldn't parse line '%s': %s" % (line, e))
    logger.debug('user %s has %s top %ss' % (username, len(results), entity_type))
    return results

def get_top_artists(username):
    """Get a user's top artists."""
    return get_chart(username, 'artist')

def get_top_tracks(username):
    """Get a user's top tracks."""
    return get_chart(username, 'track')

def get_top_albums(username):
    """Get a user's top albums."""
    return get_chart(username, 'album')

if __name__ == "__main__":
    import sys
    username = sys.argv[1]
    print get_top_artists(username)
