class __accept_all_filter:
    __contains__ = lambda self, _: True
    __repr__ = lambda self: '*'
class __accept_nothing_filter:
    __contains__ = lambda self, _: False
    __repr__ = lambda self: '()'

ANYTHING = __accept_all_filter()
"""Filter that accepts everything"""

NOTHING = __accept_nothing_filter()
"""Filter that accepts nothing"""

class AndFilter:
    """
    AND metadata filter
    
    >>> f = AndFilter([1, 2], [2, 3])
    >>> f
    ([1, 2] and [2, 3])
    >>> 0 in f
    False
    >>> 1 in f
    False
    >>> 2 in f
    True
    >>> 3 in f
    False
    """
    def __init__(self, *children):
        self.children = children
    
    def __contains__(self, x):
        for child in self.children:
            if x not in child: return False
        return True
    
    def __repr__(self):
        return '(' + ' and '.join([repr(c) for c in self.children]) + ')'

class OrFilter:
    """
    OR metadata filter
    
    >>> f = OrFilter([1, 2], [2, 3])
    >>> f
    ([1, 2] or [2, 3])
    >>> 0 in f
    False
    >>> 1 in f
    True
    >>> 2 in f
    True
    >>> 3 in f
    True
    """
    def __init__(self, *children):
        self.children = children
    
    def __contains__(self, x):
        for child in self.children:
            if x in child: return True
        return False
    
    def __repr__(self):
        return '(' + ' or '.join([repr(c) for c in self.children]) + ')'

def from_string(string):
    """Create a filter from a string.
    
        >>> from_string('_default') == None
        True
        >>> from_string('_all') == ANYTHING
        True
        >>> from_string('_none') == NOTHING
        True
        >>> from_string('a, b,c') == ['a', 'b', 'c']
        True
        >>> from_string(1)
        Traceback (most recent call last):
          ...
        AttributeError: 'int' object has no attribute 'strip'
    
    """
    if string=='_default':
        return None
    if string=='_all':
        return ANYTHING
    if string=='_none':
        return NOTHING
    return [field.strip() for field in string.strip().split(',')]
