import web
import urllib

def urlencode(query, doseq=0):
    try:
        return urllib.urlencode(query, doseq)
    except UnicodeEncodeError:
        try:
            query = query.items()
        except AttributeError:
            pass
        query = tuple((k.encode('utf8'), v.encode('utf8')) for (k, v) in query)
        return urllib.urlencode(query, doseq)

class render_mako:
    """
    "inspired" by web.contrib.render_mako
    """
    def __init__(self, _extension = 'html', *a, **kwargs):
        from mako.lookup import TemplateLookup
        self._lookup = TemplateLookup(*a, **kwargs)
        self._extension = _extension
 
    def __getattr__(self, name):
        path = name + '.' + self._extension
        def render(*args, **kwargs):
            t = self._lookup.get_template(path)
            return t.render(*args, **kwargs)
        return render

def jsonpwrap(delegate, callback_param='callback', default_callback='callback'):
    """Wrap a json render function into jsonp.
    
        >>> import jsonpickle
        >>> jsonpickle.encode(dict(x=1))
        '{"x": 1}'
        >>> jsonpwrap(jsonpickle.encode)(dict(x=1))
        'callback({"x": 1})'
    
    """
    def f(*args, **kwargs):
        try:
            params = web.input()
        except:
            params = dict()
        callback = params.get(callback_param, default_callback)
        return '%s(%s)' % (callback, delegate(*args, **kwargs))
    return f
